import argparse
import configparser
import logging
import os
import sys
from pathlib import Path
from typing import Dict

from gandi import __version__, commands
from gandi.auth import GandiAuth


def run() -> None:
    """Command line entry point"""
    args = parse_args()
    setup_logging(args)
    config = read_config(args.config)

    auth = GandiAuth(config["api_key"])
    args.func(config, auth, args)


def parse_args() -> argparse.Namespace:
    parser = argparse.ArgumentParser()
    parser.add_argument("-c", "--config", type=Path, help="path to config file")
    parser.add_argument(
        "-v", "--verbose", action="store_true", help="increase output verbosity",
    )
    parser.add_argument(
        "-V",
        "--version",
        action="version",
        version="%(prog)s {version}".format(version=__version__),
    )

    subparsers = parser.add_subparsers(title="available commands")
    commands.alias.create_command(subparsers)

    return parser.parse_args()


def setup_logging(args: argparse.Namespace) -> None:
    """Setup logging"""
    level = logging.INFO if args.verbose else logging.WARNING
    logging.basicConfig(format="%(message)s", level=level)


def read_config(conf_file: Path = None) -> Dict[str, str]:
    """Find and read configuration file"""
    if not conf_file:
        xdg_config_home = os.environ.get("XDG_CONFIG_HOME", "~/.config")
        conf_file = Path(xdg_config_home).expanduser() / "gandi" / "config"

    config = configparser.ConfigParser()
    try:
        with conf_file.open() as fil:
            config.read_file(fil)
    except FileNotFoundError:
        logging.error("Configuration file %s not found", conf_file)
        sys.exit(1)

    section = "gandi"
    keys = ("api_key", "domain", "mailbox_id")
    return {k: config.get(section, k) for k in keys}



import argparse
import json
from typing import Dict, List

import requests

from gandi.auth import GandiAuth


def create_command(parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
    subparser = parser.add_parser("alias", help="manage email aliases")
    subparser.add_argument("-l", "--list", action="store_true")
    subparser.add_argument("-a", "--add", metavar="ALIAS", type=str)
    subparser.add_argument("-d", "--delete", metavar="ALIAS", type=str)
    subparser.set_defaults(func=alias)
    return subparser


def alias(config: Dict[str, str], auth: GandiAuth, args: argparse.Namespace):
    domain = config["domain"]
    mailbox_id = config["mailbox_id"]
    url = f"https://api.gandi.net/v5/email/mailboxes/{domain}/{mailbox_id}"
    if args.list:
        print("\n".join(get_aliases(url, auth)))
    elif args.add:
        if add_alias(url, auth, args.add):
            print(f"Created alias '{alias}'")
        else:
            print(f"Alias '{alias}' already exists")

    elif args.delete:
        if delete_alias(url, auth, args.delete):
            print(f"Removed alias '{alias}'")
        else:
            print(f"Alias '{alias}' does not exist")


def get_aliases(url: str, auth: GandiAuth):
    res = requests.get(url, auth=auth)
    res.raise_for_status()
    data = json.loads(res.text)
    return data["aliases"]


def add_alias(url: str, auth: GandiAuth, alias: str):
    aliases = get_aliases(url, auth)
    if alias in aliases:
        return False

    aliases.append(alias)
    update_aliases(url, auth, aliases)
    return True


def delete_alias(url: str, auth: GandiAuth, alias: str):
    aliases = get_aliases(url, auth)
    if alias not in aliases:
        return False

    aliases.remove(alias)
    update_aliases(url, auth, aliases)
    return True


def update_aliases(url: str, auth: GandiAuth, aliases: List[str]):
    requests.patch(url, json={"aliases": aliases}, auth=auth).raise_for_status()
